
//---------------------------------------------------------------------------
// bbmobile_arduino_02
//---------------------------------------------------------------------------
// This is library for playing with BBMobile modules
// you can freely change and redistribute it but
// dont forget to add link to our website: www.bbmagic.net
//---------------------------------------------------------------------------
// To jest biblioteka obsługująca moduł BBMobile
// możesz ją dowolnie modyfikować i redystrybuować ale
// nie zapomnij tylko o zawarciu linku: www.bbmagic.net
//---------------------------------------------------------------------------
#include "Arduino.h"
#include "bbmobile_arduino_02.h"

  String bbm_buf ;
  byte bbm_conn ;

//#define DEBUG_ACK  //-you must do Serial.begin in setup() before using this

//===========================================================================================
//-------gets
//-------returns
// 0 - no Bluetooth cnnection establised
// 1 - Bluetooth cnnection is opened
//===========================================================================================
byte BBMobileIsConnected(void)
{
  return(bbm_conn) ;
}

//===========================================================================================
//-------gets
// bbm - pointer to the BBMobile stream
//-------returns
// 0 - no message from BBMobile
// 1 - there is data in bbm_buf from BBMobile
//===========================================================================================
byte BBMobileGetMessage(Stream *bbm)
{
  if(bbm->available() > 0)
  {
    bbm_buf = bbm->readStringUntil('\n') ;   
    if(bbm_buf.substring(0, 5) == ">DCON")
    {
      bbm_conn =0 ;      
      return(0) ;     
    }else if(bbm_buf.substring(0, 4) == ">CON")
    {
      bbm_conn =1 ;
      return(0) ; 
    }
//Serial.println(">>"+bbm_buf) ;      
    return(1) ;
  }
  return(0) ;    
}

//===========================================================================================
//-------gets
// bbm - pointer to the BBMobile stream
// timeout_ms - timeout in miliseconds
//-------returns
// 0 - got ACK
// 1 - timeout occured
// 2 - an error occured
// 3 - Bluetooth connection has been closed
//===========================================================================================
byte BBMobileWaitAck(Stream *bbm, int timeout_ms)
{
  do
  {
    delay(1) ;
    if(bbm->available() > 0)
    {
      bbm_buf = bbm->readStringUntil('\n') ;
      #ifdef DEBUG_ACK  
        Serial.println("ACK: " + bbm_buf) ;
      #endif  //-def DEBUG_ACK          
      if(bbm_buf.substring(0, 3) == ">OK") return(0) ;
      else if(bbm_buf.substring(0, 3) == "$ok") return(0) ;
      else if(bbm_buf.substring(0, 3) == ">HI") return(0) ;
      else if(bbm_buf.substring(0, 5) == ">DCON")
      {
        bbm_conn =0 ;              
        return(3) ;        
      }else return(2) ;
    }
  }while(timeout_ms-- >0) ;
  return(1) ;
}

//===========================================================================================
//-------gets
// bbm - pointer to the BBMobile stream
// s - string to send
//-------returns
//===========================================================================================
void BBMobileSend(Stream *bbm, String s)
{
  bbm->println(s) ;
}

//===========================================================================================
//-------gets
// bbm - pointer to the BBMobile stream
// s - string to send
//-------returns
// 0 - got ACK
// 1 - timeout occured
// 2 - an error occured
// 3 - Bluetooth connection has been closed
//===========================================================================================
byte BBMobileSendWaitAck(Stream *bbm, String s)
{
  bbm->println(s) ;
  return( BBMobileWaitAck(bbm, 1000) ) ;
}

//===========================================================================================
//-------gets
// bbm - pointer to the BBMobile stream
// j - pointer to JSON data in flash memoty
//-------returns
// 0 - got ACK
// 1 - timeout occured
// 2 - an error occured
// 3 - Bluetooth connection has been closed
//===========================================================================================
byte BBMobileSendJson(Stream *bbm, char *j)
{
  char bb ;
  while(1)
  {
    bb = pgm_read_byte(j) ;
    if(bb == 0) return ;
    bbm->write(bb) ;    
    j++ ;
  } ;
  return( BBMobileWaitAck(bbm, 3000) ) ;
}

//===========================================================================================
//-gets hex chars from string and converts to int number
//-------gets
// str - pointer to string with hex chars
//-returns
// long unsigned int number
//===========================================================================================
long unsigned int BBMobileHexstr2int(String *str)
{
  long unsigned int val ;
  byte b, n =0 ;
  
  val = 0 ;
  while(1)
  {
    b = str->charAt(n) ; 
    n++ ;    
    if (b >= '0' && b <= '9') b -= '0' ;
    else if (b >= 'a' && b <='f') b = b - 'a' + 10 ;
    else if (b >= 'A' && b <='F') b = b - 'A' + 10 ;
    else return val ;
    //-shift 4 and add
    val <<=4 ;
    val |= (b & 0xF) ;
  }
  return val ;
}

//===========================================================================================
//-gets time from BBMobile timestamp message
//-------gets
// buf - pointer to string with BBMobile timestamp message
//-returns
// local timestamp
//===========================================================================================
long unsigned int BBMobileGetLocalTime(String *pbuf)
{
  long unsigned int ts, local ;

  if(pbuf->substring(0, 3) != "$t,")  return 0 ;   //-no timestamp message
  //Serial.println(*pbuf) ;
  ts = BBMobileHexstr2int(&pbuf->substring(3)) ;     //-get timestamp
  local = BBMobileHexstr2int(&pbuf->substring(13)) ; //-get local time
  local *=60 ;                                      //-convert to seconds 
  if(pbuf->charAt(12) =='+') ts +=local ;
  else ts -=local ;
  ts -=946684800 ;                                  //-Arduino's lib starts from 01-01-2000
  Serial.println("Ts: " + String(ts, HEX)) ;
  return ts ;
}

//===========================================================================================
//-gets string value from specific field name f
//-------gets
// f - pointer to the string with BBMobile data
// ret - pointer to store string
//-returns
// 0 - everything is OK
//-2 - no such field
//-4 - field has no data
//===========================================================================================
int BBMobileGetFieldString(String *f, String *ret)
{
  int i, j ;

  i = bbm_buf.lastIndexOf(*f) ;    
  if(i < 0) return(-2) ;     //-no such field
  i += f->length() ;
  i++ ;
  if(bbm_buf.charAt(i) == '"') return(-4) ;   //-field has no data
  j = bbm_buf.indexOf('"', i) ;
  *ret =bbm_buf.substring(i, j) ;
  return( 0 ) ;
}

//===========================================================================================
//-gets int value from specific field name f
//-------gets
// f - pointer to the string with BBMobile data
// ret - pointer to store int variable
//-returns
// 0 - everything is OK
//-2 - no such field
//-4 - field has no data
//===========================================================================================
int BBMobileGetFieldInt(String *f, long int *ret)
{
  int i ;

  i = bbm_buf.lastIndexOf(*f) ;    
  if(i < 0) return(-2) ;     //-no such field
  i += f->length() ;
  i++ ;
  if(bbm_buf.charAt(i) == '"') return(-4) ;   //-field has no data
  *ret =bbm_buf.substring(i).toInt() ;
  return( 0 ) ;
}

//===========================================================================================
//-gets float value from specific field name f
//-------gets
// f - pointer to the string with BBMobile data
// ret - pointer to store float variable
//-returns
// 0 - everything is OK
//-2 - no such field
//-4 - field has no data
//===========================================================================================
int BBMobileGetFieldFloat(String *f, float *ret)
{
  int i, j ;

  i = bbm_buf.lastIndexOf(*f) ;    
  if(i < 0) return(-2) ;     //-no such field
  i += f->length() ;
  i++ ;
  if(bbm_buf.charAt(i) == '"') return(-4) ;   //-field has no data
  *ret =bbm_buf.substring(i).toFloat() ;
  return( 0 ) ;
}
